/*
 * Copyright (c) 2005, The Regents of the University of California, through
 * Lawrence Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy). All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * (2) Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * (3) Neither the name of the University of California, Lawrence Berkeley
 * National Laboratory, U.S. Dept. of Energy nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * You are under no obligation whatsoever to provide any bug fixes, patches, or
 * upgrades to the features, functionality or performance of the source code
 * ("Enhancements") to anyone; however, if you choose to make your Enhancements
 * available either publicly, or directly to Lawrence Berkeley National
 * Laboratory, without imposing a separate written license agreement for such
 * Enhancements, then you hereby grant the following license: a non-exclusive,
 * royalty-free perpetual license to install, use, modify, prepare derivative
 * works, incorporate into other computer software, distribute, and sublicense
 * such enhancements or derivative works thereof, in binary and source code
 * form.
 */
package nux.xom.sandbox;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;

import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import nu.xom.Attribute;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.NodeFactory;
import nu.xom.Serializer;
import nu.xom.Text;
import nux.xom.binary.BinaryXMLCodec;
import nux.xom.io.StreamingSerializer;
import nux.xom.io.StreamingSerializerFactory;
import nux.xom.pool.XOMUtil;

/**
 * Quick n' dirty tests for StreamingSerializer. Much more exhaustive tests can
 * be found in BinaryXMLTest.
 * 
 * @author whoschek.AT.lbl.DOT.gov
 * @author $Author: hoschek3 $
 * @version $Revision: 1.1 $, $Date: 2005/11/30 00:15:25 $
 */
class StreamingSerializerTest {

	public static void xxx() throws Exception {
		// streaming conversion of standard textual XML to bnux binary XML:
		StreamingSerializerFactory factory = new StreamingSerializerFactory();
		InputStream in = new FileInputStream("samples/data/weblog.xml");
		OutputStream out = new FileOutputStream("/tmp/weblog.xml.bnux");
		StreamingSerializer serializer = factory.createBinaryXMLSerializer(out, 0);
		NodeFactory redirector = XOMUtil.getRedirectingNodeFactory(serializer);
		
		new Builder(redirector).build(in); // performs the conversion
		
		in.close();
		out.close();
	}
	
	public static void yyy() throws Exception {
		// streaming conversion of bnux binary XML to standard textual XML:
		StreamingSerializerFactory factory = new StreamingSerializerFactory();
		InputStream in = new FileInputStream("/tmp/weblog.xml.bnux");
		OutputStream out = new FileOutputStream("/tmp/weblog.xml");
		StreamingSerializer serializer = factory.createXMLSerializer(out, "UTF-8");
		NodeFactory redirector = XOMUtil.getRedirectingNodeFactory(serializer);
		
		new BinaryXMLCodec().deserialize(in, redirector); // performs the conversion
		
		in.close();
		out.close();
	}
	
	public static void main(String[] args) throws Exception {
//		Element root = new Element("root");
//		root.addNamespaceDeclaration("xml", "http://www.w3.org/XML/1998/namespace");
////		root.addAttribute(new Attribute("xml:space", "http://www.w3.org/XML/1998/namespace", "true"));
//		Document doc = new Document(root);
		Document doc = new Builder().build(new File(args[0]));
		System.out.println("\ntoXML()="+doc.toXML());
		
		Serializer serial = new Serializer(System.out);
//		serial.setIndent(4);
		System.out.println("\nxom serial()=");
		serial.write(doc);
		
		StreamingSerializerFactory fac = new StreamingSerializerFactory();
		StreamingSerializer ser = fac.createXMLSerializer(System.out, "UTF-8");
		System.out.println("\nxom stream=");
		ser.write(doc);
//		ser.write(doc);

//		ser = fac.createStaxSerializer(createXMLStreamWriter(System.out, "UTF-8"));
//		System.out.println("\nstax stream=");
//		ser.write(doc);
	}
	
	public static void main3(String[] args) throws Exception {
		Document doc = new Builder().build(new File(args[0]));
		System.out.println("\ntoXML()="+doc.toXML());
		
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		new BinaryXMLCodec().serialize(doc, 0, out);
		Document doc2 = new BinaryXMLCodec().deserialize(out.toByteArray());
		System.out.println("\ndoc2="+doc2.toXML());
		
		out = new ByteArrayOutputStream();
		StreamingSerializerFactory fac = new StreamingSerializerFactory();
		StreamingSerializer ser = fac.createBinaryXMLSerializer(out, 0);
		ser.write(doc);
		System.out.println("\ndoc3="+ new BinaryXMLCodec().deserialize(out.toByteArray()).toXML());
		
		ser = fac.createXMLSerializer(out, "UTF-8");
		NodeFactory factory = XOMUtil.getRedirectingNodeFactory(ser);
		System.out.println("\ndocx=");
		Document docy = new Builder(factory).build(new File(args[0]));
		System.out.println("\ndocy="+docy.toXML());
		

		if (true) return;
		
		
		System.out.println("\nSerializer=");
		Serializer nser = new Serializer(System.out);
		nser.write(doc);
		
		System.out.println("\nStreamingSerializer=");
		ser = fac.createBinaryXMLSerializer(System.out, 0);
		ser.write(doc);
		out = new ByteArrayOutputStream();
		ser = fac.createBinaryXMLSerializer(out, 0);
		ser.write(doc);
		System.out.println("bnux="+ new BinaryXMLCodec().deserialize(out.toByteArray()).toXML());
		
		ser.writeXMLDeclaration();
		ser.writeStartTag(new Element("articles"));
		for (int i=0; i < 3; i++) {
			Element article = new Element("article");
			article.addAttribute(new Attribute("id", String.valueOf(i)));
			
			Element prize = new Element("prize");
			prize.appendChild(String.valueOf(i*1000));
			article.appendChild(prize);
			
			Element baz = new Element("baz");
			baz.appendChild("some text");
			prize.appendChild(baz);
			
			ser.write(article);
		}
		ser.writeEndTag();
		ser.writeEndDocument();
//		System.out.println("bnux="+ new BinaryXMLCodec().deserialize(ser.getBinaryXMLDocument()).toXML());
		
		
		ser.writeXMLDeclaration();
		ser.writeStartTag(new Element("articles"));
		for (int i=0; i < 3; i++) {
			Element article = new Element("article");
			article.addAttribute(new Attribute("id", String.valueOf(i)));
			
			Element prize = new Element("prize");
			prize.appendChild(String.valueOf(i*1000));
			article.appendChild(prize);
			
			Element quantity = new Element("quantity");
			quantity.appendChild("hello world");
			article.appendChild(quantity);
			
			ser.write(article);
		}
		ser.writeEndTag();
		ser.writeEndDocument();
//		System.out.println("bnux="+ new BinaryXMLCodec().deserialize(ser.getBinaryXMLDocument()).toXML());
		
		
		ser.writeXMLDeclaration();
		ser.writeStartTag(new Element("xx:articles", "http://example.com"));
		for (int i=0; i < 3; i++) {
			Element article = new Element("article");
			article.addAttribute(new Attribute("id", String.valueOf(i)));
			ser.writeStartTag(article);
			
			ser.writeStartTag(new Element("prize"));
			ser.write(new Text(String.valueOf(i*1000)));
			ser.writeEndTag();
			
			ser.writeStartTag(new Element("quantity"));
			System.err.println("ns="+ser.toString());
			ser.write(new Text("hello world"));
			ser.writeEndTag();
			
			ser.writeEndTag(); // close article
		}
		ser.writeEndTag(); // close articles
		ser.writeEndDocument();
//		System.out.println("bnux="+ new BinaryXMLCodec().deserialize(ser.getBinaryXMLDocument()).toXML());
		
	}

	public static void main2(String[] args) throws Exception {
		Document doc = new Builder().build(new File(args[0]));
		System.out.println("\ntoXML()="+doc.toXML());
		System.out.println("\nSerializer=");
		Serializer nser = new Serializer(System.out);
		nser.write(doc);
		
		StreamingSerializerFactory fac = new StreamingSerializerFactory();
		System.out.println("\nStreamingSerializer=");
		StreamingSerializer ser = fac.createXMLSerializer(System.out, "UTF-8");
		ser.write(doc);
		
		ser.writeXMLDeclaration();
		ser.writeStartTag(new Element("articles"));
		for (int i=0; i < 3; i++) {
			Element article = new Element("article");
			article.addAttribute(new Attribute("id", String.valueOf(i)));
			
			Element prize = new Element("prize");
			prize.appendChild(String.valueOf(i*1000));
			article.appendChild(prize);
			
			Element baz = new Element("baz");
			baz.appendChild("some text");
			prize.appendChild(baz);
			
			ser.write(article);
		}
		ser.writeEndTag();
		ser.writeEndDocument();
		
		
		ser.writeXMLDeclaration();
		ser.writeStartTag(new Element("articles"));
		for (int i=0; i < 3; i++) {
			Element article = new Element("article");
			article.addAttribute(new Attribute("id", String.valueOf(i)));
			
			Element prize = new Element("prize");
			prize.appendChild(String.valueOf(i*1000));
			article.appendChild(prize);
			
			Element quantity = new Element("quantity");
			quantity.appendChild("hello world");
			article.appendChild(quantity);
			
			ser.write(article);
		}
		ser.writeEndTag();
		ser.writeEndDocument();
		
		
		ser.writeXMLDeclaration();
		ser.writeStartTag(new Element("xx:articles", "http://example.com"));
		for (int i=0; i < 3; i++) {
			Element article = new Element("article");
			article.addAttribute(new Attribute("id", String.valueOf(i)));
			ser.writeStartTag(article);
			
			ser.writeStartTag(new Element("prize"));
			ser.write(new Text(String.valueOf(i*1000)));
			ser.writeEndTag();
			
			ser.writeStartTag(new Element("quantity"));
			System.err.println("ns="+ser.toString());
			ser.write(new Text("hello world"));
			ser.writeEndTag();
			
			ser.writeEndTag(); // close article
		}
		ser.writeEndTag(); // close articles
		ser.writeEndDocument();
	}
	
	private static XMLStreamWriter createXMLStreamWriter(OutputStream out, String encoding) throws XMLStreamException, FactoryConfigurationError {
		return XMLOutputFactory.newInstance().createXMLStreamWriter(out, encoding);
	}

}

